#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""
@author: Arkadiusz Dzięgiel
"""

from os.path import dirname
import os, re, base64, hashlib, json
import subprocess
import time
import types
import logging

class Handler(object):
    
    TYPE_GENERATED_IMAGE = 'generated_image'
    
    MODE_ABSOLUTE = 'absolute'
    MODE_APP = 'app'
    MODE_VENDOR = 'vendor'
    
    INITIAL_FILE = "python::stdin.%s"
    
    initial_scss = INITIAL_FILE % "scss"
    initial_css = INITIAL_FILE % "css"
    
    vendor_path = "vendor" # prefix webasset vendor dir
    vendor_dirs = {
        "font": "fonts",
        "image": "images",
    }
    generated_dir = "generated-images"
    
    api_version = 4
    
    def __init__(self, env, in_, out, plugins=[], imports=[], source=None):
        
        self.env = env
        self.plugins = plugins
        self.imports = imports
        
        self.logger = logging.getLogger(__name__)
        
        self.input = in_.getvalue().encode()
        self.output = out
        self.source = source
        
        self.deps = set()
    
    def get_configuration(self):
        c = {
            "environment" : ":development",
            "line_comments": True,
            "output_style" : ":expanded", #nested, expanded, compact, compressed
            
            "generated_images_path" : "/",
            "css_path" : "/dev/null",
            "sass_path" : "/dev/null",
            "plugins" : self.plugins,
            "imports" : self.imports
        }
        return c
    
    def file_to_dict(self, filepath, data, mtime=None):
        return {"mtime":mtime if mtime else os.path.getmtime(filepath), "data": base64.encodebytes(data).decode(), "hash": hashlib.md5(filepath.encode()).hexdigest(), "ext": os.path.splitext(filepath)[1][1:]}
    
    def get_path(self, path, mode, type_=None):
        
        if mode == self.MODE_ABSOLUTE:
            raise ValueError("Resolving url absolute paths is not supported")
        
        if type_ == self.TYPE_GENERATED_IMAGE:
            return self.env.resolver.resolve_output_to_path("/".join([self.generated_dir, path]), self.source)
        else:
            filepath = "/".join([self.vendor_path, self.vendor_dirs[type_], path]) if mode == self.MODE_VENDOR else path
            try:
                ret = self.env.resolver.search_for_source(filepath)
                return ret[0] if isinstance(ret, list) else ret
            except OSError:
                return None
    
    def filepath_to_dict(self, filepath, mode, type_=None):
        filepath = self.get_path(filepath, mode, type_)
        
        self.logger.debug("Trying path %s", filepath)
        
        if filepath is None or not os.path.exists(filepath):
            return None
        
        self.deps.add(filepath)
        with open(filepath,"rb") as file:
            return self.file_to_dict(filepath, file.read())
    
    def get_file(self, path, type_, mode):
        if path == self.initial_scss:
            return self.file_to_dict(self.initial_scss, self.input, time.time())
        if path == self.initial_css:
            return None
        
        return self.filepath_to_dict(path, mode, type_)
    
    def put_file(self, path, type_, data, mode):
        if not isinstance(data, bytes):
            data = data.encode()
        data = base64.decodebytes(data)
        
        if path == self.initial_css:
            self.output.write(data.decode())
        else:
            p = self.get_path(path, mode, type_)
            self.logger.info("Writting to file %s", p)
            os.makedirs(dirname(p), 0o777, True)
            with open(p, "wb") as f:
                f.write(data)
    
    def get_url(self, path, type_, mode):
        
        if mode == self.MODE_ABSOLUTE:
            return path
        
        parts = path.split("?",1)
        p = self.get_path(parts[0], mode, type_)
        if p is None:
            raise Exception("Source %s not found" % parts[0]);
        
        parts[0] = self.env.resolver.resolve_source_to_url(p, self.source)
        return "?".join(parts)
        
    def find_sprites_matching(self, path, mode):
        if mode == self.MODE_VENDOR:
            path = "/".join([self.vendor_path, self.vendor_dirs["image"], path])
        
        r = re.compile(r'^.*?('+path.replace("*", ".*")+')$')
        return ["@"+r.match(p).group(1) for p in self.env.resolver.search_for_source(path.lstrip("/"))]
    
    def run(self, proc):
        decoder = json.JSONDecoder()
        encoder = json.JSONEncoder()
        bracet = re.compile(br'^(\x1b\x5b[0-9]{1,2}m?)?({.*)$')
        while True:
            line = proc.stdout.readline()
            if not line: break
            
            m = bracet.match(line)
            if m:
                d = decoder.decode(m.group(2).decode())
                
                self.logger.debug("running %s with: %s", d["method"], ", ".join([(a if len(a)<40 else a[0:20]+"...") for a in d["args"]]))
                
                m = getattr(self, d["method"])
                ret = m(*d["args"]) if isinstance(m, types.MethodType) else m
                    
                self.logger.debug("return %s", ret)
                proc.stdin.write(encoder.encode(ret).encode() + b"\n")
                proc.stdin.flush()
            else:
                self.logger.info(line.decode().strip())

    def start(self, compass_bin):
        command = [compass_bin,'compile','--trace','-r','compass-connector']
        for i in self.plugins:
            command.extend(("-r", i))
        command.append("@%s" % (self.INITIAL_FILE % "scss"))

        with subprocess.Popen(command,
            cwd = self.env.directory,
            stdout = subprocess.PIPE,
            stdin = subprocess.PIPE,
            env = {
                "HOME": os.environ["HOME"]
            }) as proc:
            
            self.run(proc)



# -*- coding: utf-8 -*-
'''
Created on 18-06-2013

@author: Arkadiusz Dzięgiel
'''

from webassets import Environment, Bundle
from webassets_cc.filter import CompassConnectorFilter
from os.path import dirname
import unittest
from io import StringIO
import logging

class TestCompilation(unittest.TestCase):
    
    def setUp(self):
        res = dirname(__file__)+'/resources/'
        self.env = Environment(dirname(__file__)+"/out", '/media-prefix')
        self.env.cache = False
        self.env.manifest = False
        
        self.env.append_path(res+"assets", "/")
        self.env.append_path(res+"vendor", "/vendors")
        
        self.env.append_path(res, None)
        
        self.env.config["compass_bin"] = "/home/arkus/.gem/ruby/1.9.1/bin/compass"
        self.env.config["vendor_path"] = "vendor"
        
        #logging.basicConfig(level=logging.DEBUG)
        
    def get_bundle_output(self, test_name):
        f = StringIO()
        js = Bundle('scss/%s.scss' % test_name, filters=CompassConnectorFilter, output='%s.css' % test_name)
        self.env.register(test_name, js)
        js.build(output=f)
        
        return f.getvalue()

    def test_simple(self):
        o = self.get_bundle_output("test_simple")
        self.assertIn("color: red;", o)
    
    def test_simple_import(self):
        o = self.get_bundle_output("test_simple_imports")
        self.assertIn("color: red;", o)
        self.assertIn("(style.css)", o)
        self.assertIn("(/style.css)", o)

    def test_fonts(self):
        o = self.get_bundle_output("test_fonts")
        
        self.assertIn("vendor-inline-font: url('data:font/truetype;base64,AAE", o)
        self.assertIn("app-inline-font: url('data:font/truetype;base64,AAE", o)
        
        self.assertIn("url('/this.eot?#iefix')", o)
        self.assertIn("src: url('/fonts/empty.ttf');", o)
        self.assertIn("src: url('/vendors/fonts/vendor_empty.ttf');", o)
    
    def test_images(self):
        o = self.get_bundle_output("test_images")
        
        self.assertIn("vendor-image-url: url('/vendors/images/vendor_1x1.png?", o)
        self.assertIn("app-image-url: url('/images/image.png?", o)
        self.assertIn("width-app: 10px;", o)
        self.assertIn("width-vendor: 10px;", o)
        self.assertIn("image-inline: url('data:image/png;base64,iVB", o)
        self.assertIn("vendor-generated-image-busted: url('/media-prefix/generated-images/1x1.png?", o)
        self.assertIn("vendor-generated-image: url('/media-prefix/generated-images/1x1.png');", o)
        self.assertIn("generated-image-busted: url('/media-prefix/generated-images/1x1.png?", o)
        self.assertIn("generated-image: url('/media-prefix/generated-images/1x1.png');", o)
        
    def test_sprites(self):
        o = self.get_bundle_output("test_sprites")
        
        self.assertIn("background: url('/media-prefix/generated-images/sprites/something-", o)
        self.assertIn("background: url('/media-prefix/generated-images/vendor-something-", o)

    def test_zurb(self):
        self.env.config["compass_plugins"] = {"zurb-foundation":"<99"}
        o = self.get_bundle_output("test_zurb")
        self.assertIn("body", o)



#!/usr/bin/env python

import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()

requires=['webassets']

setup(
    name='webassets_compassconnector',
    version="0.2.2",
    description='Complete Compass integration for Webassets',
    long_description=README,
    author='Arkadiusz Dzięgiel',
    author_email='arkadiusz.dziegiel@glorpen.pl',
    license='GPL-3',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    install_requires=requires,
    tests_require=requires,
    test_suite="webassets_cc",
    url='https://bitbucket.org/glorpen/webassets_compassconnector',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries',
        'Topic :: Internet :: WWW/HTTP',
    ],
)



# -*- coding: utf-8 -*-
'''
Created on 11-05-2013

@author: Arkadiusz Dzięgiel
'''

from webassets.filter import Filter, option
from webassets.exceptions import FilterError
from webassets_cc import connector

try:
    from webassets.utils import hash_func
except ImportError:
    from webassets.cache import make_md5 as hash_func

class CompassConnectorFilter(Filter):
    name = 'compassconnector'
    max_debug_level = None

    options = {
        'compass': ('binary', 'COMPASS_BIN'),
        'plugins': option('COMPASS_PLUGINS', type=list),
        'vendor_path': ('relative path', 'VENDOR_PATH'),
        'imports': option('COMPASS_IMPORTS', type=list),
    }
    
    depends = None
    
    def find_dependencies(self):
        return self.depends

    def unique(self):
        hash_func(self.plugins)

    def input(self, in_, out, **kwargs):
        h = connector.Handler(self.env, in_, out, self.plugins if self.plugins else {}, self.imports if self.imports else [], kwargs["source"])
        h.vendor_path = self.vendor_path
        if not self.compass:
            raise FilterError("Compass bin path is not set")
        h.start(self.compass)
        
        self.depends = h.deps
